﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Input;
using Infragistics.Controls.Maps;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows;

namespace CustomStyle
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private const int MaxNodeCount = 50;
        //private XamNetworkNode xnn = new XamNetworkNode();
        private const int NodeWidth = 95;
        public MainWindow()
        {
            InitializeComponent();
            xnn.GlobalNodeLayouts.Add(new NetworkNodeNodeLayout
            {
                TargetTypeName = "NodeModel",
                DisplayMemberPath = "Address",
                ToolTipMemberPath = "ToolTip",
                ConnectionsMemberPath = "Connections",
                ConnectionTargetMemberPath = "Target",
                ItemTemplate = Resources["BoardTemplate"] as DataTemplate
            });

            xnn.LineEndCap = LineCap.Diamond;
            xnn.LineStartCap = LineCap.Ellipse;
            xnn.ItemsSource = new SampleTestData(MaxNodeCount).Nodes;

            this.KeyDown += MainWindow_KeyDown;
        }

        private void MainWindow_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Key.OemPeriod:
                    SelectNextNode();
                    xnn.Focus();
                    break;
                case Key.OemComma:
                    SelectPreviousNode();
                    xnn.Focus();
                    break;
            }
        }

        private void ArrangeInRow_Click(object sender, RoutedEventArgs e)
        {
            List<NetworkNodeNode> nodesList = xnn.Nodes.ToList();
            List<Point> nodeLocations = GetNodeLocations();
            for(int i=0; i<MaxNodeCount; i++)
            {
                nodesList[i].Location = nodeLocations[i];
            }
            xnn.ScaleToFit();
        }

        private List<Point> GetNodeLocations()
        {
            var locsList = new List<Point>();
            bool checkEnd = true;
            var currentLoc = new Point(100.0, 100.0);
            locsList.Add(currentLoc);
            for (int i = 1; i <= MaxNodeCount; i++)
            {
                Point nextLoc;

                if (checkEnd)
                {
                    if ((currentLoc.X + NodeWidth + 50) >= xnn.ActualWidth)
                    {
                        nextLoc = new Point(currentLoc.X, currentLoc.Y + NodeWidth + 50);
                        checkEnd = false;
                    }
                    else
                    {
                        nextLoc = new Point(currentLoc.X + NodeWidth + 50, currentLoc.Y);
                    }
                    locsList.Add(nextLoc);
                }
                else
                {
                    if ((currentLoc.X - NodeWidth - 50) <= 0)
                    {
                        nextLoc = new Point(currentLoc.X, currentLoc.Y + NodeWidth + 50);
                        checkEnd = true;
                    }
                    else
                    {
                        nextLoc = new Point(currentLoc.X - NodeWidth -50, currentLoc.Y);
                    }
                    locsList.Add(nextLoc);
                }
                currentLoc = nextLoc;
            }

            return locsList;
        }

        private void Previous_Click(object sender, RoutedEventArgs e)
        {
            SelectPreviousNode();
        }

        private void SelectPreviousNode()
        {
            if (xnn.SelectedNodes.Count > 0)
            {
                NetworkNodeNode selectedNode = xnn.SelectedNodes[0];

                if (selectedNode.IncomingConnections != null && selectedNode.IncomingConnections.ToList().Count > 0)
                {
                    NetworkNodeNode previousNode = selectedNode.IncomingConnections.First().Source;
                    xnn.SelectedNodes.Clear();
                    previousNode.IsSelected = true;
                    BringSelectedNodeIntoView();
                }
            }
        }

        private void Next_Click(object sender, RoutedEventArgs e)
        {
            SelectNextNode();
        }

        private void SelectNextNode()
        {
            if (xnn.SelectedNodes.Count > 0)
            {
                NetworkNodeNode selectedNode = xnn.SelectedNodes[0];

                if (selectedNode.OutgoingConnections != null && selectedNode.OutgoingConnections.ToList().Count > 0)
                {
                    NetworkNodeNode nextNode = selectedNode.OutgoingConnections.First().Target;
                    xnn.SelectedNodes.Clear();
                    nextNode.IsSelected = true;
                    BringSelectedNodeIntoView();
                }
            }
        }

        private void AutoArrange_Click(object sender, RoutedEventArgs e)
        {
            xnn.UpdateNodeArrangement();
            xnn.ScaleToFit();
        }

        private void BringSelectedNodeIntoView()
        {
            NetworkNodeNode selectedNode = xnn.SelectedNodes[0];
            bool adjustWindowRectangle = false;
            double targetLeft = xnn.WindowRect.Left;
            double targetRight = xnn.WindowRect.Right;
            double targetTop = xnn.WindowRect.Top;
            double targetBottom = xnn.WindowRect.Bottom;

            if (selectedNode.Location.X < xnn.WindowRect.Left)
            {
                double diff = xnn.WindowRect.Left - selectedNode.Location.X;
                targetLeft = selectedNode.Location.X;
                targetRight = xnn.WindowRect.Right - diff;
                adjustWindowRectangle = true;
            }

            if (selectedNode.Location.X + 120 > xnn.WindowRect.Right)
            {
                double diff = selectedNode.Location.X + 120 - xnn.WindowRect.Right;
                targetRight = selectedNode.Location.X + 120;
                targetLeft = xnn.WindowRect.Left + diff;
                adjustWindowRectangle = true;
            }

            if (selectedNode.Location.Y < xnn.WindowRect.Top)
            {
                double diff = xnn.WindowRect.Top - selectedNode.Location.Y;
                targetTop = selectedNode.Location.Y;
                targetBottom = xnn.WindowRect.Bottom - diff;
                adjustWindowRectangle = true;
            }

            if (selectedNode.Location.Y + 40 > xnn.WindowRect.Bottom)
            {
                double diff = selectedNode.Location.Y + 40 - xnn.WindowRect.Bottom;
                targetBottom = selectedNode.Location.Y + 40;
                targetTop = xnn.WindowRect.Top + diff;
                adjustWindowRectangle = true;
            }

            if (adjustWindowRectangle)
            {
                xnn.WindowRect = new Rect(new Point(targetLeft, targetTop), new Point(targetRight, targetBottom));
            }
        }

        private void ClearSelection_Click(object sender, RoutedEventArgs e)
        {
            xnn.SelectedNodes.Clear();
            xnn.ActiveNode = null;
        }
    }


    public class SampleTestData
    {
        public ObservableCollection<NodeModel> Nodes { get; set; }

        public SampleTestData(int nodeCount)
        {
            Nodes = new ObservableCollection<NodeModel>();
            for (int i = 1; i <= nodeCount; i++)
            {
                var nm = new NodeModel
                {
                    Address = i,
                    ToolTip = "Main node at address " + i,
                    Connections = new ObservableCollection<ConnectionModel>(),
                };

                nm.Port1Node.Id = i + "_P1";
                nm.Port1Node.ToolTip = "sub node at port 1";
                nm.Port2Node.Id = i + "_P2";
                nm.Port2Node.ToolTip = "sub node at port 2";
                Nodes.Add(nm);

            }

            for (int i = 1; i < nodeCount; i++)
            {
                NodeModel source = Nodes[i-1];
                NodeModel target = Nodes[i];
                var connectionModel = new ConnectionModel { Target = target };
                {
                    source.Connections.Add(connectionModel);
                }
            }
        }
    }





    public class SubNodeModel
    {
        public string Id { get; set; }
        public string ToolTip { get; set; } 
    }

    public class NodeModel : INotifyPropertyChanged
    {
        public NodeModel()
        {
            Port1Node = new SubNodeModel();
            Port2Node = new SubNodeModel();
        }

        private int _address;
        public int Address
        {
            get { return _address; }
            set
            {
                if (value != _address)
                {
                    _address = value;
                    NotifyPropertyUpdated("Address");
                }
            }
        }

        private string _toolTip;
        public string ToolTip
        {
            get { return _toolTip; }
            set
            {
                if (value != _toolTip)
                {
                    _toolTip = value;
                    NotifyPropertyUpdated("ToolTip");
                }
            }
        }

        private ObservableCollection<ConnectionModel> _connections;
        public ObservableCollection<ConnectionModel> Connections
        {
            get { return _connections; }
            set
            {
                if (value != _connections)
                {
                    _connections = value;
                    NotifyPropertyUpdated("Connections");
                }
            }
        }

        public SubNodeModel Port1Node { get; set; }
        public SubNodeModel Port2Node { get; set; }

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void NotifyPropertyUpdated(string propertyName)
        {
            var handler = PropertyChanged;

            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }

    public class ConnectionModel : INotifyPropertyChanged
    {
        private NodeModel _target;
        public NodeModel Target
        {
            get { return _target; }
            set
            {
                if (value != _target)
                {
                    _target = value;
                    NotifyPropertyUpdated("Target");
                }
            }
        }

        #region Implementation of INotifyPropertyChanged

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void NotifyPropertyUpdated(string propertyName)
        {
            var handler = PropertyChanged;

            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion
    }


}
